@@ -21,10 +21,13 @@
 #include "kmines_debug.h"
 #include <QGraphicsScene>
 #include <QGraphicsSceneMouseEvent>
+#include <QRandomGenerator>
 
 #include "cellitem.h"
 #include "borderitem.h"
 
+#include "settings.h"
+
 MineFieldItem::MineFieldItem(KGameRenderer* renderer)
     : m_leftButtonPos(-1,-1), m_midButtonPos(-1,-1), m_gameOver(false),
       m_emulatingMidButton(false), m_renderer(renderer)
@@ -44,7 +47,7 @@ void MineFieldItem::resetMines()
     }
 
     m_flaggedMinesCount = 0;
-    emit flaggedMinesCountChanged(m_flaggedMinesCount);
+    Q_EMIT flaggedMinesCountChanged(m_flaggedMinesCount);
 }
 
 
@@ -108,7 +111,7 @@ void MineFieldItem::initField( int numRows, int numCols, int numMines )
 
     adjustItemPositions();
     m_flaggedMinesCount = 0;
-    emit flaggedMinesCountChanged(m_flaggedMinesCount);
+    Q_EMIT flaggedMinesCountChanged(m_flaggedMinesCount);
 }
 
 void MineFieldItem::generateField(int clickedIdx)
@@ -127,9 +130,10 @@ void MineFieldItem::generateField(int clickedIdx)
     // (it will be empty if none of surrounding items holds mine)
     QList<CellItem*> neighbForClicked = adjacentItemsFor(fp.first, fp.second);
 
+    QRandomGenerator random(QRandomGenerator::global()->generate());
     while(minesToPlace != 0)
     {
-        randomIdx = m_randomSeq.getLong( m_numRows*m_numCols );
+        randomIdx = random.bounded( m_numRows*m_numCols );
         item = m_cells.at(randomIdx);
         if(!item->hasMine()
            && neighbForClicked.indexOf(item) == -1
@@ -144,12 +148,10 @@ void MineFieldItem::generateField(int clickedIdx)
             continue;
     }
 
-    foreach(int idx, cellsWithMines)
-    {
+    for (int idx : qAsConst(cellsWithMines)) {
         FieldPos rc = rowColFromIndex(idx);
-        QList<CellItem*> neighbours = adjacentItemsFor(rc.first, rc.second);
-        foreach( CellItem *item, neighbours )
-        {
+        const QList<CellItem*> neighbours = adjacentItemsFor(rc.first, rc.second);
+        for (CellItem *item : neighbours) {
             if(!item->hasMine())
                 item->setDigit( item->digit()+1 );
         }
@@ -264,11 +266,13 @@ void MineFieldItem::resizeToFitInRect(const QRectF& rect)
 
     m_cellSize = static_cast<int>(size);
 
-    foreach( CellItem* item, m_cells )
+    for (CellItem* item : qAsConst(m_cells)) {
         item->setRenderSize(QSize(m_cellSize, m_cellSize));
+    }
 
-    foreach( BorderItem *item, m_borders)
+    for (BorderItem *item : qAsConst(m_borders)) {
         item->setRenderSize(QSize(m_cellSize, m_cellSize));
+    }
 
     adjustItemPositions();
 }
@@ -283,8 +287,7 @@ void MineFieldItem::adjustItemPositions()
             itemAt(row,col)->setPos((col+1)*m_cellSize, (row+1)*m_cellSize);
         }
 
-    foreach( BorderItem* item, m_borders )
-    {
+    for (BorderItem* item : qAsConst(m_borders)) {
         item->setPos( item->col()*m_cellSize, item->row()*m_cellSize );
     }
 }
@@ -309,11 +312,10 @@ bool MineFieldItem::onItemRevealed(int row, int col)
 void MineFieldItem::revealEmptySpace(int row, int col)
 {
     // recursively reveal neighbour cells until we find cells with digit
-    QList<FieldPos> list = adjacentRowColsFor(row,col);
+    const QList<FieldPos> list = adjacentRowColsFor(row,col);
     CellItem *item = nullptr;
 
-    foreach( const FieldPos& pos, list )
-    {
+    for (const FieldPos& pos : list) {
         // first is row, second is col
         item = itemAt(pos);
         if(item->isRevealed() || item->isFlagged() || item->isQuestioned())
@@ -349,8 +351,9 @@ void MineFieldItem::mousePressEvent( QGraphicsSceneMouseEvent *ev )
         return;
     }
 
-    m_emulatingMidButton = ( (ev->buttons() & Qt::LeftButton) && (ev->buttons() & Qt::RightButton) );
-    bool midButtonPressed = (ev->button() == Qt::MidButton || m_emulatingMidButton );
+    bool useFastExplore = Settings::exploreWithLeftClickOnNumberCells();
+    m_emulatingMidButton = ( useFastExplore ? ( (ev->buttons() & Qt::LeftButton) && ( itemUnderMouse->isRevealed() ) ) : ( (ev->buttons() & Qt::LeftButton) && (ev->buttons() & Qt::RightButton) ) );
+    bool midButtonPressed = (ev->button() == Qt::MiddleButton || m_emulatingMidButton );
 
     if(midButtonPressed)
     {
@@ -358,9 +361,8 @@ void MineFieldItem::mousePressEvent( QGraphicsSceneMouseEvent *ev )
         // undo press that was made by LeftClick. in other cases it won't hurt :)
         itemUnderMouse->undoPress();
 
-        QList<CellItem*> neighbours = adjacentItemsFor(row,col);
-        foreach(CellItem* item, neighbours)
-        {
+        const QList<CellItem*> neighbours = adjacentItemsFor(row,col);
+        for (CellItem* item : neighbours) {
             if(!item->isFlagged() && !item->isQuestioned() && !item->isRevealed())
                 item->press();
             m_midButtonPos = qMakePair(row,col);
@@ -391,9 +393,10 @@ void MineFieldItem::mouseReleaseEvent( QGraphicsSceneMouseEvent * ev)
         // and return
         if(m_midButtonPos.first != -1)
         {
-            QList<CellItem*> neighbours = adjacentItemsFor(m_midButtonPos.first,m_midButtonPos.second);
-            foreach(CellItem *item, neighbours)
+            const QList<CellItem*> neighbours = adjacentItemsFor(m_midButtonPos.first,m_midButtonPos.second);
+            for (CellItem *item : neighbours) {
                 item->undoPress();
+            }
             m_midButtonPos = qMakePair(-1,-1);
             m_emulatingMidButton = false;
         }
@@ -408,24 +411,24 @@ void MineFieldItem::mouseReleaseEvent( QGraphicsSceneMouseEvent * ev)
 
     CellItem* itemUnderMouse = itemAt(row,col);
 
-    bool midButtonReleased = (ev->button() == Qt::MidButton || m_emulatingMidButton);
+    bool midButtonReleased = (ev->button() == Qt::MiddleButton || m_emulatingMidButton);
 
     if( midButtonReleased )
     {
         m_midButtonPos = qMakePair(-1,-1);
 
-        QList<CellItem*> neighbours = adjacentItemsFor(row,col);
+        const QList<CellItem*> neighbours = adjacentItemsFor(row,col);
         if(!itemUnderMouse->isRevealed())
         {
-            foreach(CellItem *item, neighbours)
+            for (CellItem *item : neighbours) {
                 item->undoPress();
+            }
             return;
         }
 
         int numFlags = 0;
         int numMines = 0;
-        foreach(CellItem *item, neighbours)
-        {
+        for (CellItem *item : neighbours) {
             if(item->isFlagged())
                 numFlags++;
             if(item->hasMine())
@@ -433,8 +436,7 @@ void MineFieldItem::mouseReleaseEvent( QGraphicsSceneMouseEvent * ev)
         }
         if(numFlags == numMines && numFlags != 0)
         {
-            foreach(CellItem *item, neighbours)
-            {
+            for (CellItem *item : neighbours) {
                 if(!item->isRevealed()) // revealing only unrevealed ones
                 {
                     // force=true to omit Pressed check
@@ -450,8 +452,9 @@ void MineFieldItem::mouseReleaseEvent( QGraphicsSceneMouseEvent * ev)
         }
         else
         {
-            foreach(CellItem *item, neighbours)
+            for (CellItem *item : neighbours) {
                 item->undoPress();
+            }
         }
     }
     else if(ev->button() == Qt::LeftButton && (ev->buttons() & Qt::RightButton) == false)
@@ -474,7 +477,7 @@ void MineFieldItem::mouseReleaseEvent( QGraphicsSceneMouseEvent * ev)
             {
                 m_firstClick = false;
                 generateField( row*m_numCols + col );
-                emit firstClickDone();
+                Q_EMIT firstClickDone();
             }
 
             itemUnderMouse->release();
@@ -496,7 +499,7 @@ void MineFieldItem::mouseReleaseEvent( QGraphicsSceneMouseEvent * ev)
                 m_flaggedMinesCount++;
             else
                 m_flaggedMinesCount--;
-            emit flaggedMinesCountChanged(m_flaggedMinesCount);
+            Q_EMIT flaggedMinesCountChanged(m_flaggedMinesCount);
         }
     }
 }
@@ -512,7 +515,7 @@ void MineFieldItem::mouseMoveEvent( QGraphicsSceneMouseEvent *ev )
     if( row < 0 || row >= m_numRows || col < 0 || col >= m_numCols )
         return;
 
-    bool midButtonPressed = ((ev->buttons() & Qt::MidButton) ||
+    bool midButtonPressed = ((ev->buttons() & Qt::MiddleButton) ||
                             ( (ev->buttons() & Qt::LeftButton) && (ev->buttons() & Qt::RightButton) ) );
 
     if(midButtonPressed)
@@ -521,15 +524,17 @@ void MineFieldItem::mouseMoveEvent( QGraphicsSceneMouseEvent *ev )
            (m_midButtonPos.first != row || m_midButtonPos.second != col))
         {
             // un-press previously pressed cells
-            QList<CellItem*> prevNeighbours = adjacentItemsFor(m_midButtonPos.first,
-                                                              m_midButtonPos.second);
-            foreach(CellItem *item, prevNeighbours)
+            const QList<CellItem*> prevNeighbours = adjacentItemsFor(m_midButtonPos.first,
+                                                                     m_midButtonPos.second);
+            for (CellItem *item : prevNeighbours) {
                    item->undoPress();
+            }
 
             // and press current neighbours
-            QList<CellItem*> neighbours = adjacentItemsFor(row,col);
-            foreach(CellItem *item, neighbours)
+            const QList<CellItem*> neighbours = adjacentItemsFor(row,col);
+            for (CellItem *item : neighbours) {
                 item->press();
+            }
 
             m_midButtonPos = qMakePair(row,col);
         }
@@ -548,8 +553,7 @@ void MineFieldItem::mouseMoveEvent( QGraphicsSceneMouseEvent *ev )
 
 void MineFieldItem::revealAllMines()
 {
-    foreach( CellItem* item, m_cells )
-    {
+    for (CellItem* item : qAsConst(m_cells)) {
         if( (item->isFlagged() && !item->hasMine()) || (!item->isFlagged() && item->hasMine()) )
         {
             item->reveal();
@@ -575,12 +579,11 @@ bool MineFieldItem::onItemRevealed(CellItem* item)
 bool MineFieldItem::checkLost()
 {
     // for loss...
-    foreach( CellItem* item, m_cells )
-    {
+    for (CellItem* item : qAsConst(m_cells)) {
         if(item->isExploded())
         {
             m_gameOver = true;
-            emit gameOver(false);
+            Q_EMIT gameOver(false);
             return true;
         }
     }
@@ -595,8 +598,7 @@ bool MineFieldItem::checkWon()
     if(m_numUnrevealed == m_minesCount)
     {
         // mark not flagged cells (if any) with flags
-        foreach( CellItem* item, m_cells )
-        {
+        for (CellItem* item : qAsConst(m_cells)) {
             if( item->isQuestioned() )
                 item->mark();
             if( !item->isRevealed() && !item->isFlagged() )
@@ -604,8 +606,8 @@ bool MineFieldItem::checkWon()
         }
         m_gameOver = true;
         // now all mines should be flagged, notify about this
-        emit flaggedMinesCountChanged(m_minesCount);
-        emit gameOver(true);
+        Q_EMIT flaggedMinesCountChanged(m_minesCount);
+        Q_EMIT gameOver(true);
         return true;
     }
     return false;
@@ -635,10 +637,11 @@ QList<FieldPos> MineFieldItem::adjacentRowColsFor(int row, int col)
 
 QList<CellItem*> MineFieldItem::adjacentItemsFor(int row, int col)
 {
-    QList<FieldPos > rowcolList = adjacentRowColsFor(row,col);
+    const QList<FieldPos > rowcolList = adjacentRowColsFor(row,col);
     QList<CellItem*> resultingList;
-    foreach( const FieldPos& pos, rowcolList )
+    for (const FieldPos& pos : rowcolList) {
         resultingList.append( itemAt(pos) );
+    }
     return resultingList;
 }
 
